@@ -72,11 +72,11 @@ __all__ = [
     "convert_image", "convert_images",
     "convert_image_to_numpy", "convert_image_to_pil",
     "ImageConvert", "ImageOutput", "ImageOverlay", "ImageOverlayFilter",
-    "ImageReadFile", "ImageReadZMQ", "ImageResize",
+    "ImageReadFile", "ImageReadZMQ", "ImageResize", "ImageSquareCenterCrop",
     "ImageWriteFile", "ImageWriteZMQ"
 ]
 
-_LOGGER = aiko.get_logger(__name__)
+_LOGGER = aiko.process.logger(__name__)
 
 _CV2_IMPORTED = False
 try:
@@ -157,7 +157,7 @@ def convert_images(images, media_type):
 class ImageConvert(aiko.PipelineElement):
     def __init__(self, context: aiko.ContextPipelineElement):
         context.set_protocol("image_convert:0")
-        context.get_implementation("PipelineElement").__init__(self, context)
+        context.call_init(self, "PipelineElement", context)
 
     def process_frame(self, stream, images) -> Tuple[aiko.StreamEvent, dict]:
         media_type, _ = self.get_parameter("media_type", None)
@@ -171,7 +171,7 @@ class ImageConvert(aiko.PipelineElement):
 class ImageOutput(aiko.PipelineElement):
     def __init__(self, context: aiko.ContextPipelineElement):
         context.set_protocol("image_output:0")
-        context.get_implementation("PipelineElement").__init__(self, context)
+        context.call_init(self, "PipelineElement", context)
 
     def process_frame(self, stream, images) -> Tuple[aiko.StreamEvent, dict]:
         return aiko.StreamEvent.OKAY, {"images": images}
@@ -186,7 +186,7 @@ class ImageOutput(aiko.PipelineElement):
 class ImageOverlay(aiko.PipelineElement):
     def __init__(self, context):
         context.set_protocol("image_overlay:0")
-        context.get_implementation("PipelineElement").__init__(self, context)
+        context.call_init(self, "PipelineElement", context)
         self.color = (0, 255, 255)
         self.font = cv2.FONT_HERSHEY_SIMPLEX
         self.font_scale = 0.75
@@ -256,7 +256,7 @@ class ImageOverlay(aiko.PipelineElement):
 class ImageOverlayFilter(aiko.PipelineElement):
     def __init__(self, context):
         context.set_protocol("image_overlay_filter:0")
-        context.get_implementation("PipelineElement").__init__(self, context)
+        context.call_init(self, "PipelineElement", context)
 
     def process_frame(self, stream, overlay)  \
         -> Tuple[aiko.StreamEvent, dict]:
@@ -300,7 +300,7 @@ class ImageOverlayFilter(aiko.PipelineElement):
 class ImageReadFile(aiko.DataSource):  # PipelineElement
     def __init__(self, context: aiko.ContextPipelineElement):
         context.set_protocol("image_read_file:0")
-        context.get_implementation("PipelineElement").__init__(self, context)
+        context.call_init(self, "PipelineElement", context)
 
     def process_frame(self, stream, paths) -> Tuple[aiko.StreamEvent, dict]:
         stream.variables["timestamps"] = [stream.frame_id * (1 / 25)]
@@ -333,7 +333,7 @@ class ImageReadFile(aiko.DataSource):  # PipelineElement
 class ImageReadZMQ(aiko.DataSource):  # PipelineElement
     def __init__(self, context: aiko.ContextPipelineElement):
         context.set_protocol("image_read_zmq:0")
-        context.get_implementation("PipelineElement").__init__(self, context)
+        context.call_init(self, "PipelineElement", context)
 
     def process_frame(self, stream, records) -> Tuple[aiko.StreamEvent, dict]:
         stream.variables["timestamps"] = [stream.frame_id * (1 / 25)]
@@ -358,6 +358,7 @@ class ImageReadZMQ(aiko.DataSource):  # PipelineElement
         return aiko.StreamEvent.OKAY, {"images": images}
 
 # --------------------------------------------------------------------------- #
+# TODO: Combine ImageResize and ImageSquareCenterCrop PipelineElements
 # TODO: Add logic for using different backends (opencv,...) or
 #           different input types (np.ndarray, ...)
 #
@@ -370,7 +371,7 @@ class ImageReadZMQ(aiko.DataSource):  # PipelineElement
 class ImageResize(aiko.PipelineElement):
     def __init__(self, context: aiko.ContextPipelineElement):
         context.set_protocol("image_resize:0")
-        context.get_implementation("PipelineElement").__init__(self, context)
+        context.call_init(self, "PipelineElement", context)
 
     def process_frame(self, stream, images) -> Tuple[aiko.StreamEvent, dict]:
         resolution, found = self.get_parameter("resolution")
@@ -389,6 +390,44 @@ class ImageResize(aiko.PipelineElement):
             images_resized.append(image_resized)
         return aiko.StreamEvent.OKAY, {"images": images_resized}
 
+# --------------------------------------------------------------------------- #
+# TODO: Combine ImageResize and ImageSquareCenterCrop PipelineElements
+#       ImageResize should be able to handle both Pillow and NumPy array
+#       ImageResize should have a "square_centre_crop=boolean" parameter
+
+def square_center_crop(image):  # Either Pillow or NumPy array Image
+    if isinstance(image, Image.Image):
+        width, height = image.size
+        is_pil = True
+    elif isinstance(image, np.ndarray):
+        height, width = image.shape[:2]
+        is_pil = False
+    else:
+        raise TypeError(
+            f"square_center_crop() requires PIL.Image or np.ndarray, "
+            f"but received type {type(image)}"
+        )
+
+    side = min(width, height)  # Maintain shortest side
+    left = (width - side) // 2
+    top  = (height - side) // 2
+
+    if is_pil:
+        return image.crop((left, top, left + side, top + side))
+    else:
+        return image[top : top + side, left : left + side, ...]
+
+class ImageSquareCenterCrop(aiko.PipelineElement):
+    def __init__(self, context: aiko.ContextPipelineElement):
+        context.set_protocol("image_square_center_crop:0")
+        context.call_init(self, "PipelineElement", context)
+
+    def process_frame(self, stream, images) -> Tuple[aiko.StreamEvent, dict]:
+        images_cropped = []
+        for image in images:
+            images_cropped.append(square_center_crop(image))
+        return aiko.StreamEvent.OKAY, {"images": images_cropped}
+
 # --------------------------------------------------------------------------- #
 # ImageWriteFile is a DataTarget that writes images to files
 #
@@ -399,7 +438,7 @@ class ImageResize(aiko.PipelineElement):
 class ImageWriteFile(aiko.DataTarget):  # PipelineElement
     def __init__(self, context: aiko.ContextPipelineElement):
         context.set_protocol("image_write_file:0")
-        context.get_implementation("PipelineElement").__init__(self, context)
+        context.call_init(self, "PipelineElement", context)
 
     def process_frame(self, stream, images) -> Tuple[aiko.StreamEvent, dict]:
         for image in images:
@@ -434,7 +473,7 @@ class ImageWriteFile(aiko.DataTarget):  # PipelineElement
 class ImageWriteZMQ(aiko.DataTarget):  # PipelineElement
     def __init__(self, context: aiko.ContextPipelineElement):
         context.set_protocol("image_write_zmq:0")
-        context.get_implementation("PipelineElement").__init__(self, context)
+        context.call_init(self, "PipelineElement", context)
 
     def process_frame(self, stream, images) -> Tuple[aiko.StreamEvent, dict]:
         media_type = "image"                            # TODO: "image/zip" ?
