@@ -73,7 +73,7 @@ __all__ = [
     "VideoWriteFile", "VideoWriteFiles"
 ]
 
-_LOGGER = aiko.get_logger(__name__)
+_LOGGER = aiko.process.logger(__name__)
 
 _DEFAULT_MINUTE_RANGE = "*"   # "*" or specific minute range to capture "00-01"
 
@@ -113,7 +113,7 @@ except ModuleNotFoundError:  # TODO: Optional warning flag
 class VideoOutput(aiko.PipelineElement):
     def __init__(self, context: aiko.ContextPipelineElement):
         context.set_protocol("video_output:0")
-        context.get_implementation("PipelineElement").__init__(self, context)
+        context.call_init(self, "PipelineElement", context)
 
     def process_frame(self, stream, images) -> Tuple[aiko.StreamEvent, dict]:
         return aiko.StreamEvent.OKAY, {"images": images}
@@ -131,7 +131,7 @@ class VideoOutput(aiko.PipelineElement):
 class VideoReadFile(aiko.DataSource):  # PipelineElement
     def __init__(self, context: aiko.ContextPipelineElement):
         context.set_protocol("video_read_file:0")
-        context.get_implementation("PipelineElement").__init__(self, context)
+        context.call_init(self, "PipelineElement", context)
 
     def start_stream(self, stream, stream_id):
         stream.variables["video_capture"] = None
@@ -204,7 +204,7 @@ class VideoReadFile(aiko.DataSource):  # PipelineElement
 class VideoSample(aiko.PipelineElement):
     def __init__(self, context):
         context.set_protocol("video_sample:0")
-        context.get_implementation("PipelineElement").__init__(self, context)
+        context.call_init(self, "PipelineElement", context)
 
     def process_frame(self, stream, images) -> Tuple[aiko.StreamEvent, dict]:
         sample_rate, _ = self.get_parameter("sample_rate", 1)
@@ -221,7 +221,7 @@ class VideoSample(aiko.PipelineElement):
 class VideoShow(aiko.PipelineElement):
     def __init__(self, context):
         context.set_protocol("video_show:0")
-        context.get_implementation("PipelineElement").__init__(self, context)
+        context.call_init(self, "PipelineElement", context)
 
 #   def slider_handler(self, value):
 #       print(f"slider: {value}")
@@ -269,7 +269,7 @@ class VideoShow(aiko.PipelineElement):
 class VideoWriteFile(aiko.DataTarget):  # PipelineElement
     def __init__(self, context: aiko.ContextPipelineElement):
         context.set_protocol("video_write_file:0")
-        context.get_implementation("PipelineElement").__init__(self, context)
+        context.call_init(self, "PipelineElement", context)
 
     def start_stream(self, stream, stream_id):
         stream_event, diagnostic = super().start_stream(stream, stream_id)
@@ -311,6 +311,10 @@ class VideoWriteFile(aiko.DataTarget):  # PipelineElement
     def process_frame(self, stream, images) -> Tuple[aiko.StreamEvent, dict]:
         self.logger.debug(f"{self.my_id()}")
 
+        if images and not isinstance(images[0], np.ndarray):
+            diagnostic = "Image media_type must be a numpy array"
+            return aiko.StreamEvent.ERROR, {"diagnostic": diagnostic}
+
         if stream.variables["video_writer"]:
             video_writer = stream.variables["video_writer"]
         else:
@@ -343,7 +347,7 @@ class VideoWriteFile(aiko.DataTarget):  # PipelineElement
 class VideoWriteFiles(aiko.PipelineElement):
     def __init__(self, context: aiko.ContextPipelineElement):
         context.set_protocol("video_write_files:0")
-        context.get_implementation("PipelineElement").__init__(self, context)
+        context.call_init(self, "PipelineElement", context)
 
     def start_stream(self, stream, stream_id):
         stream.variables["last_minute"] = 0
