@@ -202,6 +202,10 @@ PIPELINE_HOOK_PROCESS_ELEMENT_POST = "pipeline.process_element_post:"
 _PIPELINE_HOOK_PROCESS_ELEMENT_POST = PIPELINE_HOOK_PROCESS_ELEMENT_POST+"0"
 PIPELINE_HOOK_PROCESS_FRAME = "pipeline.process_frame:"
 _PIPELINE_HOOK_PROCESS_FRAME = PIPELINE_HOOK_PROCESS_FRAME+"0"
+PIPELINE_HOOK_PROCESS_FRAME_COMPLETE = "pipeline.process_frame_complete:"
+_PIPELINE_HOOK_PROCESS_FRAME_COMPLETE = PIPELINE_HOOK_PROCESS_FRAME_COMPLETE+"0"
+PIPELINE_HOOK_DESTROY_STREAM = "pipeline.destroy_stream:"
+_PIPELINE_HOOK_DESTROY_STREAM = PIPELINE_HOOK_DESTROY_STREAM+"0"
 
 _GRACE_TIME = 60  # seconds
 _LOGGER = aiko.logger(__name__)
@@ -434,7 +438,7 @@ class PipelineElementImpl(PipelineElement):
         if context.protocol == "*":
             context.set_protocol(
                 PROTOCOL_PIPELINE if self.is_pipeline else PROTOCOL_ELEMENT)
-        context.get_implementation("Actor").__init__(self, context)
+        context.call_init(self, "Actor", context)
 
     #  "log_level" parameter overrides "AIKO_LOG_LEVEL" environment variable
         log_level, found = self.get_parameter(
@@ -676,7 +680,7 @@ class PipelineImpl(Pipeline):
 
         self.pipeline_graph = None
         self.actor = context.get_implementation("Actor")    # _WINDOWS
-        context.get_implementation("PipelineElement").__init__(self, context)
+        context.call_init(self, "PipelineElement", context)
         self.logger.info(f"MQTT topic: {self.topic_in}")
 
         self.share["definition_pathname"] = context.definition_pathname
@@ -697,6 +701,8 @@ class PipelineImpl(Pipeline):
         self.add_hook(_PIPELINE_HOOK_PROCESS_ELEMENT)
         self.add_hook(_PIPELINE_HOOK_PROCESS_ELEMENT_POST)
         self.add_hook(_PIPELINE_HOOK_PROCESS_FRAME)
+        self.add_hook(_PIPELINE_HOOK_PROCESS_FRAME_COMPLETE)
+        self.add_hook(_PIPELINE_HOOK_DESTROY_STREAM)
 
         self.pipeline_graph = self._create_pipeline_graph(context.definition)
         self.share["element_count"] = self.pipeline_graph.element_count
@@ -1012,7 +1018,8 @@ class PipelineImpl(Pipeline):
                             stream_id, Graph.path_remote(stream.graph_path),
                             parameters, grace_time, None, self.topic_in)
         finally:
-            stream.lock.release()
+            if stream.lock.in_use():
+                stream.lock.release()
             self._disable_thread_local("create_stream()")
         return True
 
@@ -1093,21 +1100,23 @@ class PipelineImpl(Pipeline):
                         stream.state = StreamState.ERROR
 
             # Notify listeners that the stream has stopped
-            stop_state = stream.state
-            if stop_state >= StreamState.RUN:
-                stop_state = StreamState.STOP
+            if stream.state >= StreamState.RUN:
+                stream.state = StreamState.STOP
+            self.run_hook(_PIPELINE_HOOK_DESTROY_STREAM,
+                lambda: {"stream": stream,
+                         "diagnostic": diagnostic})
             stream_info = {
                 "stream_id": stream.stream_id,
                 "frame_id": stream.frame_id,
-                "state": stop_state}
+                "state": stream.state}
             if stream.queue_response:
                 stream.queue_response.put((stream_info, diagnostic))
             if stream.topic_response:
                 actor = get_actor_mqtt(stream.topic_response, Pipeline)
                 actor.process_frame_response(stream_info, diagnostic)
         finally:
+            stream.lock.release()
             if use_thread_local:
-                stream.lock.release()
                 self._disable_thread_local("destroy_stream()")
 
         stream_lease = self.stream_leases[stream_id]
@@ -1312,6 +1321,7 @@ class PipelineImpl(Pipeline):
                         self._process_metrics_start(metrics)
                         self.run_hook(_PIPELINE_HOOK_PROCESS_ELEMENT, lambda: {
                             "element_name": element_name,
+                            "element": element,
                             "stream": stream,
                             "inputs": inputs})
                         try:
@@ -1340,6 +1350,7 @@ class PipelineImpl(Pipeline):
                         self.run_hook(_PIPELINE_HOOK_PROCESS_ELEMENT_POST,
                             lambda: {
                                 "element_name": element_name,
+                                "element": element,
                                 "stream": stream, "stream_event": stream_event,
                                 "frame_data_out": frame_data_out})
                         stream.set_state(self._process_stream_event(
@@ -1390,6 +1401,10 @@ class PipelineImpl(Pipeline):
                     "stream_id": stream.stream_id,
                     "frame_id": stream.frame_id,
                     "state": stream.state}
+                self.run_hook(_PIPELINE_HOOK_PROCESS_FRAME_COMPLETE,
+                    lambda: {
+                        "stream": stream,
+                        "frame_data_out": frame_data_out})
                 if stream.queue_response:
                     stream.queue_response.put((stream_info, frame_data_out))
                 elif stream.topic_response:
@@ -1411,7 +1426,8 @@ class PipelineImpl(Pipeline):
                 del stream.frames[stream.frame_id]
             if frame_complete and stream.frame_id in stream.frames:
                 del stream.frames[stream.frame_id]
-            stream.lock.release()
+            if stream.lock.in_use():
+                stream.lock.release()
             self._disable_thread_local("process_frame()")
         return True
 
@@ -1693,7 +1709,7 @@ class PipelineImpl(Pipeline):
 
 class PipelineRemote(PipelineElement):
     def __init__(self, context):
-        context.get_implementation("PipelineElement").__init__(self, context)
+        context.call_init(self, "PipelineElement", context)
         self.set_remote_absent(True)
 
     def create_stream(self, stream_id, graph_path=None,
