@@ -17,7 +17,7 @@
 #
 # class ServiceTestImpl(ServiceTest):
 #     def __init__(self, context):
-#         context.get_implementation("Service").__init__(self, context)
+#         context.call_init(self, "Service", context)
 #
 #     def test(self):
 #         print("ServiceTestImpl.test() invoked")
@@ -50,6 +50,8 @@
 # ~~~~~
 # - BUG: Provide filtered Services to "service_change_handler"
 #
+# * Combine ServiceFields and ServiceFilter into a single data structure ?
+#
 # - Service Hooks should be optional, whilst Actor Hooks are baked in
 #
 # - Consider using @dataclass, as per "pipeline.py"
@@ -98,6 +100,7 @@ from collections import OrderedDict  # All OrderedDict operations are O(1)
 import time
 
 from aiko_services.main import *
+from aiko_services.main.utilities import get_hostname
 
 __all__ = [
     "ServiceFields", "ServiceFilter", "ServiceProtocol",
@@ -222,6 +225,8 @@ class ServiceFilter:
     def __init__(self, topic_paths="*",
         name="*", protocol="*", transport="*", owner="*", tags="*"):
 
+        name = name if name else get_hostname()
+
         self.topic_paths = topic_paths
         self.name = name
         self.protocol = protocol
@@ -230,9 +235,9 @@ class ServiceFilter:
         self.tags = tags
 
     def __repr__(self):
-        return f"{self.topic_paths}, {self.name}, "    \
-               f"{self.protocol}, {self.transport}, "  \
-               f"{self.owner}, {self.tags}"
+        return f"({self.topic_paths} {self.name} "    \
+               f"{self.protocol} {self.transport} "  \
+               f"{self.owner} ({" ".join(self.tags)}))"
 
     def summary(self):
         summary  = "" if self.topic_paths == "*" else f", {self.topic_paths}"
@@ -546,7 +551,7 @@ class Service(ServiceProtocolInterface, Hooks):  # TODO: Make Hooks be optional
 # --------------------------------------------------------------------------- #
 
 class ServiceImpl(Service):
-    def __init__(self, context):
+    def __init__(self, context, register_service=True):
 
     # TODO: Move name, protocol, tags, topic_path, transport into ServiceFields
         self.time_started = time.monotonic()
@@ -554,7 +559,11 @@ class ServiceImpl(Service):
         self.protocol = context.protocol
         self._tags = context.tags
         self.transport = context.transport
-        aiko.process.add_service(self)  # Initializes service_id and topic_path
+
+        self.service_id = None
+        self.topic_path = None
+        if register_service:
+            aiko.process.add_service(self)  # Assigns service_id and topic_path
 
         self._registrar_handler_function = None
         self.topic_control = f"{self.topic_path}/control"
