@@ -12,14 +12,21 @@ from uuid import UUID, uuid4
 
 import aiko_services as aiko
 import yaml
+from aiko_services import Stream
 from aiko_services import process as aiko_process
 from aiko_services.main import aiko as aiko_main
-from aiko_services.main import generate, parse
+from aiko_services.main.pipeline import (
+    _PIPELINE_HOOK_DESTROY_STREAM,
+    _PIPELINE_HOOK_PROCESS_ELEMENT_POST,
+    _PIPELINE_HOOK_PROCESS_FRAME_COMPLETE,
+)
+from aiko_services.main.utilities import generate, parse
 
 from highlighter import MachineAgentVersion
 from highlighter.client.agents import create_pipeline_instance, update_pipeline_instance
 from highlighter.client.base_models.base_models import UserType
 from highlighter.client.gql_client import HLClient
+from highlighter.client.json_tools import HLJSONEncoder
 from highlighter.client.tasks import (
     Task,
     TaskStatus,
@@ -27,9 +34,8 @@ from highlighter.client.tasks import (
     lease_tasks_from_steps,
     update_task,
 )
-from highlighter.core.data_models import DataSample
 from highlighter.core.database.database import Database
-from highlighter.core.hl_base_model import GQLBaseModel
+from highlighter.core.gql_base_model import GQLBaseModel
 from highlighter.core.shutdown import runtime_stop_event
 
 __all__ = [
@@ -110,6 +116,14 @@ def _validate_path(s) -> Optional[Path]:
         return None
 
 
+class AgentError(Exception):
+    """Exception raised for fatal errors in the Highlighter agent.
+
+    This exception indicates a serious problem within the agent
+    that prevents normal operation
+    """
+
+
 class HLAgent:
 
     def __init__(
@@ -156,6 +170,7 @@ class HLAgent:
             else:
                 raise SystemExit(f"pipeline_definition '{pipeline_definition}' id does not exist")
 
+        self.logger.debug(f"Pipeline Definition: {json.dumps(definition_dict, indent=2, cls=HLJSONEncoder)}")
         self._dump_definition(definition_dict, pipeline_path)
 
         parsed_definition = aiko.PipelineImpl.parse_pipeline_definition(pipeline_path)
@@ -168,6 +183,41 @@ class HLAgent:
         )
         pipeline = aiko.compose_instance(aiko.PipelineImpl, init_args)
 
+        def after_process_frame_hook(hook_name, component, logger, variables, options):
+            capability = variables["element"]
+            # Data source capabilities don't have a "record_outputs" method
+            if hasattr(capability, "record_outputs"):
+                frame_data_out = variables["frame_data_out"]
+                stream_id = variables["stream"].stream_id
+                capability.record_outputs(stream_id, frame_data_out, variables["stream_event"])
+
+        pipeline.add_hook_handler(_PIPELINE_HOOK_PROCESS_ELEMENT_POST, after_process_frame_hook)
+
+        self.frame_complete_hooks = {}
+
+        def frame_complete_hook(hook_name, component, logger, variables, options):
+            stream_id = variables["stream"].stream_id
+            if stream_id in self.frame_complete_hooks:
+                self.frame_complete_hooks[stream_id](
+                    stream=variables["stream"], frame_data_out=variables["frame_data_out"]
+                )
+
+        pipeline.add_hook_handler(_PIPELINE_HOOK_PROCESS_FRAME_COMPLETE, frame_complete_hook)
+
+        self.destroy_stream_hooks = {}
+
+        def destroy_stream_hook(hook_name, component, logger, variables, options):
+            stream_id = variables["stream"].stream_id
+            if stream_id in self.destroy_stream_hooks:
+                self.destroy_stream_hooks[stream_id](
+                    stream=variables["stream"], diagnostic=variables["diagnostic"]
+                )
+                del self.destroy_stream_hooks[stream_id]
+            if stream_id in self.frame_complete_hooks:
+                del self.frame_complete_hooks[stream_id]
+
+        pipeline.add_hook_handler(_PIPELINE_HOOK_DESTROY_STREAM, destroy_stream_hook)
+
         self.pipeline = pipeline
         self.pipeline_definition = parsed_definition
         self.timeout_secs = timeout_secs
@@ -189,11 +239,11 @@ class HLAgent:
 
     def _dump_definition(self, pipeline_def: Dict, path: Path):
         with path.open("w") as f:
-            json.dump(pipeline_def, f, sort_keys=True, indent=2)
+            json.dump(pipeline_def, f, sort_keys=True, indent=2, cls=HLJSONEncoder)
 
     def run_in_new_thread(self, mqtt_connection_required=False):
         if aiko_process.running:
-            raise RuntimeError("Aiko event-loop thread is already running")
+            raise AgentError("Aiko event-loop thread is already running")
         self.aiko_event_loop_thread = Thread(
             target=self.pipeline.run,
             daemon=True,
@@ -240,7 +290,7 @@ class HLAgent:
                             self.pipeline.stream_leases[stream_id] = lease
                         self.logger.debug(f"Enabling thread_local for stream {stream_id}")
                         self.pipeline._enable_thread_local("HLAgent.stop_all_streams()", stream_id)
-                        if not graceful:
+                        if not graceful and stream_id in self.pipeline.stream_leases:
                             del self.pipeline.stream_leases[stream_id]
                 except Exception as e:
                     self.logger.warn(
@@ -260,6 +310,7 @@ class HLAgent:
             self.aiko_event_loop_thread.join(timeout=5)  # Wait for current frame to be processed
             if self.aiko_event_loop_thread.is_alive():
                 self.logger.warning("Aiko event-loop thread not stopped")
+        self.db.close()
 
     def get_head_data_source_capability(self):
         head_capability = [x for x in self.pipeline.pipeline_graph][0]
@@ -277,68 +328,39 @@ class HLAgent:
         }
         return self.pipeline.process_frame(stream, frame_data)
 
-    def _set_agent_data_source_stream_parameters(self, data_sources, stream_parameters):
+    def create_frame(
+        self, stream: Stream, frame_data: dict, frame_id: int | None = None, graph_path: str | None = None
+    ):
+        return self.pipeline.create_frame(stream, frame_data, frame_id=frame_id, graph_path=graph_path)
 
-        data_source_capabilities = self.get_data_source_capabilities()
-        if len(data_source_capabilities) == 1:
-            data_source_capability_name = data_source_capabilities[0].name
-        elif len(data_source_capabilities) > 1:
-            raise NotImplementedError(
-                f"hl agent start cannot yet support Agents with multiple DataSource Capabilities, got: {data_source_capabilities}"
-            )
-        else:
-            raise NotImplementedError(
-                "hl agent start cannot yet support Agents with no DataSource Capabilities"
-            )
+    def create_stream(
+        self, stream_id, *, frame_complete_hook=None, destroy_stream_hook=None, parameters=None, **kwargs
+    ):
+        if frame_complete_hook:
+            self.frame_complete_hooks[stream_id] = frame_complete_hook
+        if destroy_stream_hook:
+            self.destroy_stream_hooks[stream_id] = destroy_stream_hook
+        parameters = parameters if parameters else {}
+        parameters["database"] = self.db
+        try:
+            return self.pipeline.create_stream(stream_id=stream_id, parameters=parameters, **kwargs)
+        except Exception:
+            if stream_id in self.frame_complete_hooks:
+                del self.frame_complete_hooks[stream_id]
+            if stream_id in self.destroy_stream_hooks:
+                del self.destroy_stream_hooks[stream_id]
+            raise
 
-        input_name = f"{data_source_capability_name}.data_sources"
-        stream_parameters[input_name] = data_sources
-        return stream_parameters
-
-    # ToDo: Now that I am not accepting cli passed stream params. Is this needed,
-    # or is it just handled by the aiko pipeline
-    def parse_stream_parameters(self, agent_definition) -> dict:
-
-        stream_parameters = {}
-        for node in self.pipeline.pipeline_graph.nodes():
-            node_name = node.name
-
-            # If default_stream_parameters is not present, we are likely
-            # working with an aiko.PipelineElement. In this case, we can assume
-            # that parameter validation is handled manually.
-            if not hasattr(node.element, "default_stream_parameters"):
-                continue
-
-            # Start with in code params
-            default_stream_parameters = node.element.default_stream_parameters()
-
-            # Overwite with global pipeline definition params
-            global_pipeline_definition_params = {
-                k: v
-                for k, v in agent_definition.parameters.items()
-                if k in node.element.DefaultStreamParameters.model_fields
-            }
-            default_stream_parameters.update(global_pipeline_definition_params)
-
-            # Overwite with per element pipeline definition paras
-            element_definition = [e for e in agent_definition.elements if e.name == node_name][0]
-            pipeline_element_definition_params = {
-                k.replace(f"{node_name}.", ""): v
-                for k, v in element_definition.parameters.items()
-                if k.replace(f"{node_name}.", "") in node.element.DefaultStreamParameters.model_fields
-            }
-            node.element.parameters = pipeline_element_definition_params
-            default_stream_parameters.update(pipeline_element_definition_params)
-
-            ele_stream_parameters = node.element.DefaultStreamParameters(
-                **default_stream_parameters
-            ).model_dump()
-
-            stream_parameters.update(
-                {f"{element_definition.name}.{k}": v for k, v in ele_stream_parameters.items()}
-            )
-
-        return stream_parameters
+    def destroy_stream(
+        self, stream_id, graceful=False, use_thread_local=True, diagnostic={}, with_lock=False
+    ):
+        return self.pipeline.destroy_stream(
+            stream_id=stream_id,
+            graceful=graceful,
+            use_thread_local=use_thread_local,
+            diagnostic=diagnostic,
+            with_lock=with_lock,
+        )
 
     def process_data_sources(
         self,
@@ -348,18 +370,12 @@ class HLAgent:
         queue_response_max_size=100,
         timeout_secs=120,
     ):
-        stream_parameters: dict = self.parse_stream_parameters(
-            self.pipeline_definition,
-        )
-
+        stream_parameters = {}
         stream_parameters["database"] = self.db
 
         if data_sources:
             data_source_sexp = SExpression.encode(None, data_sources)
-            stream_parameters = self._set_agent_data_source_stream_parameters(
-                data_source_sexp, stream_parameters
-            )
-
+            stream_parameters["data_sources"] = data_source_sexp
         queue_response = Queue(maxsize=queue_response_max_size)
         self.pipeline.create_stream(stream_id, parameters=stream_parameters, queue_response=queue_response)
 
@@ -392,6 +408,8 @@ class HLAgent:
                 break
 
     def loop_over_process_frame(self, stream_id, frame_datas, queue_response):
+        from highlighter.core.data_models import DataSample
+
         # This function can be removed once
         # the issue it's solving is resolved.
         # See to function's doc string for more info
@@ -400,11 +418,7 @@ class HLAgent:
         if isinstance(frame_datas, dict):
             frame_datas = [frame_datas]
 
-        stream_parameters: dict = self.parse_stream_parameters(
-            self.pipeline_definition,
-        )
-
-        self.pipeline.create_stream(stream_id, parameters=stream_parameters, queue_response=queue_response)
+        self.pipeline.create_stream(stream_id, parameters={}, queue_response=queue_response)
         for frame_id, frame in enumerate(frame_datas):
             if runtime_stop_event.is_set() or not self.enable_create_stream:
                 break
@@ -424,18 +438,22 @@ class HLAgent:
             self.pipeline.process_frame(stream, {"data_samples": data_samples})
         self.pipeline.destroy_stream(stream_id)
 
-    def poll_for_tasks_loop(self, step_id: Union[str, UUID], allow_non_machine_user: bool = False):
-        if not self.enable_create_stream:
-            raise RuntimeError("Cannot process tasks when creating streams is disabled")
-
+    def check_user_is_machine(self):
         current_user = HLClient.get_client().current_user(return_type=UserType)
-        if current_user.machine_agent_version_id is None and not allow_non_machine_user:
-            raise RuntimeError(
+        if current_user.machine_agent_version_id is None:
+            raise AgentError(
                 "Running agent as non-machine user. To run the agent as a machine user, use "
                 "`hl agent create-token` and set HL_WEB_GRAPHQL_API_TOKEN with the returned value before running `hl agent start. "
                 "To run the agent as the current user, pass `allow_non_machine_user=True`."
             )
 
+    def poll_for_tasks_loop(self, step_id: Union[str, UUID], allow_non_machine_user: bool = False):
+        if not self.enable_create_stream:
+            raise AgentError("Cannot process tasks when creating streams is disabled")
+
+        if not allow_non_machine_user:
+            self.check_user_is_machine()
+
         step_id = UUID(step_id)
 
         # Report running agent to hl web
@@ -477,71 +495,60 @@ class HLAgent:
                         lease_sec=self.task_lease_duration_secs,
                     )
                 for task in tasks:
-                    self._process_task(task)
+
+                    def frame_complete_hook(stream, frame_data_out):
+                        if "task_leased_until" not in stream.variables:
+                            stream.variables["task_leased_until"] = task.leased_until.timestamp()
+                        if (
+                            stream.variables["task_leased_until"]
+                            < datetime.now(timezone.utc).timestamp() + TASK_RE_LEASE_EXPIRY_BUFFER_SECONDS
+                        ):
+                            # Re-lease task
+                            self.logger.info(f"Extending lease for task {stream.stream_id}")
+                            stream.variables["task_leased_until"] = lease_task(
+                                client=HLClient.get_client(),
+                                task_id=stream.stream_id,
+                                lease_sec=self.task_lease_duration_secs,
+                            ).leased_until
+
+                    def destroy_stream_hook(stream, diagnostic):
+                        if stream.state == aiko.StreamEvent.STOP:
+                            self.logger.info(f"Completed task {stream.stream_id}")
+                            update_task(
+                                client=HLClient.get_client(),
+                                task_id=stream.stream_id,
+                                status=TaskStatus.SUCCESS,
+                            )
+                        elif stream.state == aiko.StreamEvent.ERROR:
+                            self.logger.error(
+                                f"Error in task {stream.stream_id}: {diagnostic.get('diagnostic')}"
+                            )
+                            update_task(
+                                client=HLClient.get_client(),
+                                task_id=stream.stream_id,
+                                status=TaskStatus.FAILED,
+                                message=diagnostic.get("diagnostic"),
+                            )
+
+                    self.logger.info(f"Processing task {task.id}")
+                    update_task(
+                        client=HLClient.get_client(),
+                        task_id=task.id,
+                        status=TaskStatus.RUNNING,
+                    )
+                    self.create_stream(
+                        stream_id=task.id,
+                        parameters=task.parameters,
+                        grace_time=STREAM_TIMEOUT_GRACE_TIME_SECONDS,
+                        frame_complete_hook=frame_complete_hook,
+                        destroy_stream_hook=destroy_stream_hook,
+                    )
                 if len(tasks) == 0:
                     time.sleep(self.task_polling_period_secs)
         except Exception as e:
             update_pipeline_instance(self.pipeline_instance_id, status="FAILED", message=str(e))
             raise e
 
-    def _process_task(self, task: Task):
-        self.logger.info(f"Processing task {task.id}")
-        stream_id = task.id
-        stream_result_queue = Queue()
-        update_task(
-            client=HLClient.get_client(),
-            task_id=task.id,
-            status=TaskStatus.RUNNING,
-        )
-        self.pipeline.create_stream(
-            stream_id,
-            parameters=task.parameters,
-            queue_response=stream_result_queue,
-            grace_time=STREAM_TIMEOUT_GRACE_TIME_SECONDS,
-        )
-        while not runtime_stop_event.is_set():
-            try:
-                stream_info, frame_data = stream_result_queue.get(timeout=self.timeout_secs)
-            except Empty:
-                diagnostic = (
-                    f"Timeout: Agent has not produced a value for more than {self.timeout_secs} seconds"
-                )
-                update_task(
-                    client=HLClient.get_client(),
-                    task_id=task.id,
-                    status=TaskStatus.FAILED,
-                    message=diagnostic,
-                )
-                raise ValueError(diagnostic)
-            if stream_info["state"] == aiko.StreamEvent.STOP:
-                self.logger.info(f"Completed task {task.id}")
-                update_task(
-                    client=HLClient.get_client(),
-                    task_id=task.id,
-                    status=TaskStatus.SUCCESS,
-                )
-                break
-            if stream_info["state"] == aiko.StreamEvent.ERROR:
-                self.logger.error(f"Error in task {task.id}: {frame_data.get('diagnostic')}")
-                update_task(
-                    client=HLClient.get_client(),
-                    task_id=task.id,
-                    status=TaskStatus.FAILED,
-                    message=frame_data.get("diagnostic"),
-                )
-                break
-            if (
-                task.leased_until.timestamp()
-                < datetime.now(timezone.utc).timestamp() + TASK_RE_LEASE_EXPIRY_BUFFER_SECONDS
-            ):
-                # Re-lease task
-                self.logger.info(f"Extending lease for task {task.id}")
-                task = lease_task(
-                    client=HLClient.get_client(),
-                    task_id=task.id,
-                    lease_sec=self.task_lease_duration_secs,
-                )
-
 
 def set_mock_aiko_messager():
     # ToDo: Chat with Andy about if this is a requirement. The issue is
