@@ -10,20 +10,13 @@ __all__ = ["ImageOverlay", "ImageResize"]
 
 
 class ImageResize(Capability):
-    class DefaultStreamParameters(Capability.DefaultStreamParameters):
+    class StreamParameters(Capability.StreamParameters):
         width: int = 0  # If zero, will maintain aspect ratio with respect to height
         height: int = 0  # If zero, will maintain aspect ratio with respect to width
 
-    @property
-    def width(self) -> int:
-        return self._get_parameter("width")[0]
-
-    @property
-    def height(self) -> int:
-        return self._get_parameter("height")[0]
-
-    def process_frame(self, data_samples: List[DataSample]) -> Tuple[StreamEvent, Optional[Dict]]:
-        data_samples = [ds.resize(width=self.width, height=self.height) for ds in data_samples]
+    def process_frame(self, stream, data_samples: List[DataSample]) -> Tuple[StreamEvent, Optional[Dict]]:
+        parameters = self.stream_parameters(stream.stream_id)
+        data_samples = [ds.resize(width=parameters.width, height=parameters.height) for ds in data_samples]
         return StreamEvent.OKAY, {"data_samples": data_samples}
 
 
