@@ -6,14 +6,12 @@ from datetime import datetime, timedelta, timezone
 from enum import Enum
 from logging import getLogger
 from pathlib import Path
-from typing import Any, Dict, Generator, Iterable, Optional, Tuple, Union
+from typing import Any, Dict, Generator, Iterable, Optional, Tuple
 from urllib.parse import urlparse
-from uuid import UUID
 
 import av
 import numpy as np
 from av.error import FFmpegError
-from sqlmodel import Session
 
 from highlighter.agent.capabilities.base_capability import IMAGE, TEXT, VIDEO
 from highlighter.agent.capabilities.data_sample_processor import (
@@ -26,7 +24,7 @@ from highlighter.client.io import (
     _pil_open_image_path,
     _pil_open_image_url,
 )
-from highlighter.core.data_models import DataSample
+from highlighter.core.data_models.data_sample import DataSample
 from highlighter.core.enums import ContentTypeEnum
 from highlighter.io.url import is_url_scheme
 
@@ -161,14 +159,6 @@ class TextDataSource(DataSourceCapability):
 
     stream_media_type = TEXT
 
-    class DefaultStreamParameters(DataSourceCapability.DefaultStreamParameters):
-        byte_encoding: Optional[str] = "utf-8"
-
-    @property
-    def byte_encoding(self) -> str:
-        value, _ = self._get_parameter("byte_encoding")
-        return value
-
     def get_text_frame_generator(self, stream):
 
         if "source_paths_generator" in stream.variables:
@@ -279,22 +269,18 @@ class JsonArrayDataSource(DataSourceCapability):
 
     stream_media_type = TEXT
 
-    class DefaultStreamParameters(DataSourceCapability.DefaultStreamParameters):
+    class StreamParameters(DataSourceCapability.StreamParameters):
         key: str = ""
 
-    @property
-    def key(self) -> str:
-        value, _ = self._get_parameter("key")
-        return value
-
     def get_json_array_frame_generator(self, stream):
+        parameters = self.stream_parameters(stream.stream_id)
 
         if "source_paths_generator" in stream.variables:
             source_urls = [path for path, _ in stream.variables["source_paths_generator"]]
             for frame_data in JsonArrayFrameIterator(
                 source_buffers=None,
                 source_urls=source_urls,
-                key=self.key,
+                key=parameters.key,
                 logger=self.logger,
             ):
                 yield frame_data
@@ -302,7 +288,7 @@ class JsonArrayDataSource(DataSourceCapability):
             for frame_data in JsonArrayFrameIterator(
                 source_buffers=stream.variables["source_buffers"],
                 source_urls=None,
-                key=self.key,
+                key=parameters.key,
                 logger=self.logger,
             ):
                 yield frame_data
@@ -388,9 +374,8 @@ class ImageFrameIterator:
             img = np.array(img, dtype=np.uint8)
 
         data_sample = DataSample(
-            # data_file_id=self.ds.id,
             content=img,
-            content_type="image",
+            content_type=ContentTypeEnum.IMAGE,
             stream_frame_index=0,
             media_frame_index=0,
         )
@@ -410,32 +395,56 @@ class ImageDataSource(DataSourceCapability):
 
     stream_media_type = IMAGE
 
-    class DefaultStreamParameters(DataSourceCapability.DefaultStreamParameters):
+    class StreamParameters(DataSourceCapability.StreamParameters):
         output_type: OutputType = OutputType.numpy
 
-    @property
-    def output_type(self) -> OutputType:
-        value, _ = self._get_parameter("output_type")
-        return value
-
     def get_image_frame_generator(self, stream):
+        parameters = self.stream_parameters(stream.stream_id)
+        if parameters.record != RecordMode.OFF and parameters.database is None:
+            raise ValueError("Missing 'database', required when recording")
 
         if "source_paths_generator" in stream.variables:
-            for frame_data in ImageFrameIterator(
+            ifi = ImageFrameIterator(
                 source_buffers=None,
                 source_urls=stream.variables["source_paths_generator"],
-                output_type=self.output_type,
+                output_type=parameters.output_type,
                 logger=self.logger,
-            ):
-                yield frame_data
+            )
+            processor = DataSampleProcessor(
+                account_uuid=parameters.account_uuid,
+                content_type=ContentTypeEnum.IMAGE,
+                data_source_uuid=parameters.data_source_uuid,
+                iterator=ifi,
+                output_filename_template=parameters.output_filename_template,
+                output_folder=parameters.output_folder,
+                record=parameters.record,  # <-- enable persistence
+                samples_per_file=parameters.samples_per_file,  # batch size
+                session_factory=lambda: parameters.database.get_session(),
+                writer_opts=parameters.writer_opts,  # forwarded to DataFile.save_local() writer
+            )
+            for data_sample in processor:
+                yield data_sample
         elif "source_buffers" in stream.variables:
-            for frame_data in ImageFrameIterator(
+            ifi = ImageFrameIterator(
                 source_buffers=stream.variables["source_buffers"],
                 source_urls=None,
-                output_type=self.output_type,
+                output_type=parameters.output_type,
                 logger=self.logger,
-            ):
-                yield frame_data
+            )
+            processor = DataSampleProcessor(
+                account_uuid=parameters.account_uuid,
+                content_type=ContentTypeEnum.IMAGE,
+                data_source_uuid=parameters.data_source_uuid,
+                iterator=ifi,
+                output_filename_template=parameters.output_filename_template,
+                output_folder=parameters.output_folder,
+                record=parameters.record,  # <-- enable persistence
+                samples_per_file=parameters.samples_per_file,  # batch size
+                session_factory=lambda: self.init_parameters.database.get_session(),
+                writer_opts=parameters.writer_opts,  # forwarded to DataFile.save_local() writer
+            )
+            for data_sample in processor:
+                yield data_sample
         elif "records" in stream.variables:
             raise NotImplementedError("#### ToDo records ####")
         else:
@@ -468,6 +477,8 @@ class VideoReader:
         sample_fps (Optional[float]): Target frames per second for sampling. If None, uses original video FPS.
         sample_frame_idxs (Optional[Iterable[int]]): Target frames by index. If None, uses original video FPS.
         resize_to (Optional[Tuple[int, int]]): If set, each frame is resized via ffmpeg to (width, height) before conversion to RGB/array.
+        original_fps (Optional[float]): Override for the original FPS when automatic detection fails. If None, uses stream metadata.
+        open_opts (Optional[Dict[str, Any]]): Additional options to pass to av.open(). Useful for RTSP transport settings, error handling, etc.
         logger (Optional[logging.Logger]): Custom logger instance. If None, a default logger is created.
 
     Raises:
@@ -476,6 +487,7 @@ class VideoReader:
 
     Attributes:
         bit_rate (float): Estimated bitrate of the video in kbps.
+        crop_zone (Optional[Tuple[int, int, int, int]]): zone to crop in the video
         duration_ms (float): Duration of the video in milliseconds.
         frame_interval (int): Number of frames to skip between sampled frames.
         height (int): Height of the video frames in pixels.
@@ -499,14 +511,18 @@ class VideoReader:
         source_url=None,
         output_type: OutputType = OutputType.numpy,
         sample_fps: Optional[float] = None,
+        crop_zone: Optional[Tuple[int, int, int, int] | Tuple[float, float, float, float]] = None,
         sample_frame_idxs: Optional[Iterable[int]] = None,
         resize_to: Optional[Tuple[int, int]] = None,
+        original_fps: Optional[float] = None,
+        open_opts: Optional[Dict[str, Any]] = None,
         logger=None,
     ):
         self.logger = logger if logger is not None else getLogger("VideoReader")
         self.output_type = output_type
         self.source_url = source_url
         self.source_buffer = source_buffer
+        self._crop_zone = crop_zone
         self._resize_to = resize_to
 
         if (sample_frame_idxs is not None) and (sample_fps is not None):
@@ -514,6 +530,8 @@ class VideoReader:
 
         self._sample_frame_idxs = sample_frame_idxs
         self._sample_fps = sample_fps
+        self._original_fps_override = original_fps
+        self._open_opts = open_opts or {}
 
         if self.source_url is not None:
             self.logger.info(
@@ -532,8 +550,18 @@ class VideoReader:
 
     @property
     def original_fps(self) -> float:
-        """Orignal frame rate of the video."""
-        return float(self._stream.average_rate or self._stream.rate)
+        """Original frame rate of the video."""
+        if self._original_fps_override is not None:
+            return self._original_fps_override
+
+        stream_fps = self._stream.average_rate or self._stream.rate
+        if stream_fps is not None:
+            return float(stream_fps)
+
+        raise ValueError(
+            "Cannot determine original FPS: stream metadata unavailable and no original_fps override provided. "
+            "For RTSP cameras without rate metadata, please specify original_fps parameter."
+        )
 
     @property
     def total_frames(self):
@@ -585,7 +613,10 @@ class VideoReader:
 
     def _init_video_container(self):
         try:
-            self._container = av.open(self._source, mode="r")
+            if self._open_opts:
+                self._container = av.open(self._source, mode="r", options=self._open_opts)
+            else:
+                self._container = av.open(self._source, mode="r")
             self.start_time = datetime.now(tz=timezone.utc)
             if isinstance(self._source, str):
                 self._original_source_url = self._source
@@ -669,6 +700,8 @@ class VideoReader:
                 self._current_frame_idx = -1
                 raise StopIteration
 
+        crop_zone = self._crop_zone
+
         resize = self._resize_to
         if resize is not None and (frame.width, frame.height) != resize:
             rw, rh = resize
@@ -691,11 +724,17 @@ class VideoReader:
 
         data_sample = DataSample(
             content=frame_img,
-            content_type="image",
+            content_type=ContentTypeEnum.IMAGE,
             recorded_at=self.start_time + timedelta(seconds=float(timestamp)),
             stream_frame_index=self._current_frame_idx,
             media_frame_index=self._current_frame_idx,
         )
+
+        # TODO: should we crop and then resize?
+        if crop_zone:
+            data_samples = data_sample.crop_content([crop_zone], as_data_sample=True)
+            data_sample = data_samples[0]
+
         return data_sample
 
     def __del__(self):
@@ -792,54 +831,33 @@ class VideoDataSource(DataSourceCapability):
 
     stream_media_type = VIDEO
 
-    class DefaultStreamParameters(DataSourceCapability.DefaultStreamParameters):
+    class InitParameters(DataSourceCapability.InitParameters):
+        crop_zone: Optional[Tuple[int, int, int, int] | Tuple[float, float, float, float]] = None
+        open_opts: Optional[Dict[str, Any]] = None
+        original_fps: Optional[float] = None
         output_type: OutputType = OutputType.numpy
-        data_source_uuid: Optional[UUID] = None
-        account_uuid: Optional[UUID] = None
-        samples_per_file: int = DEFAULT_SAMPLES_PER_FILE
-        sample_fps: Optional[float] = None
-        resize_to: Optional[str] = None
-        record: RecordMode = RecordMode.OFF
         recording_content_type: ContentTypeEnum = ContentTypeEnum.VIDEO
-        writer_opts: Dict = {}
-        data_file_id: Optional[UUID] = None
+        resize_to: Optional[str] = None
+        sample_fps: Optional[float] = None
+        samples_per_file: int = DEFAULT_SAMPLES_PER_FILE
 
-    @property
-    def output_type(self) -> OutputType:
-        value, _ = self._get_parameter("output_type")
-        return value
-
-    @property
-    def sample_fps(self) -> float:
-        value, _ = self._get_parameter("sample_fps")
-        return value
+    class StreamParameters(InitParameters):
+        pass
 
     def get_video_frame_generator(self, stream):
-        database = stream.parameters.get("database", None)
-        record, found_record = self._get_parameter("record")
-        if found_record and record != RecordMode.OFF and database is None:
-            raise ValueError("Missing 'database', required when recording")
-        data_source_uuid = self._get_parameter("data_source_uuid")[0]
-        account_uuid = self._get_parameter("account_uuid")[0]
-        samples_per_file = self._get_parameter("samples_per_file")[0]
-        sample_fps = self._get_parameter("sample_fps")[0]
-        resize_to_string = self._get_parameter("resize_to")[0]
-        record = self._get_parameter("record")[0]
-        recording_content_type = self._get_parameter("recording_content_type")[0]
-        writer_opts_overrides = self._get_parameter("writer_opts")[0]
-        resize_to_string = self._get_parameter("resize_to")[0]
+        parameters = self.stream_parameters(stream.stream_id)
+        if parameters.record != RecordMode.OFF and parameters.database is None:
+            raise ValueError("Missing 'database' parameter, required when recording")
+        resize_to_string = parameters.resize_to
         if resize_to_string is not None:
             resize_to = parse_resize_to(resize_to_string)
         else:
             resize_to = None
 
-        data_file_id = self._get_parameter("data_file_id")[0]
-        # Use the existing data_file_id if we're processing a Highlighter video
-        # data_file_id = None
-        video_data_file_ids, found = self._get_parameter("video_data_file_ids")
-        if not found:
-            if "video_data_file_ids" in stream.variables:
-                video_data_file_ids = stream.variables["video_data_file_ids"]
+        data_file_id = None
+        video_data_file_ids = None
+        if "video_data_file_ids" in stream.variables:
+            video_data_file_ids = stream.variables["video_data_file_ids"]
         if video_data_file_ids is not None:
             assert len(video_data_file_ids) == 1
             data_file_id = video_data_file_ids[0]
@@ -851,22 +869,27 @@ class VideoDataSource(DataSourceCapability):
                 vfi = VideoReader(
                     source_buffer=None,
                     source_url=str(source_url),
-                    sample_fps=sample_fps,
+                    sample_fps=parameters.sample_fps,
+                    crop_zone=parameters.crop_zone,
                     resize_to=resize_to,
-                    output_type=self.output_type,
+                    original_fps=parameters.original_fps,
+                    open_opts=parameters.open_opts,
+                    output_type=parameters.output_type,
                     logger=self.logger,
                 )
-                writer_opts = {"frame_rate": vfi._fps, **writer_opts_overrides}
+                writer_opts = {"frame_rate": vfi._fps, **parameters.writer_opts}
                 processor = DataSampleProcessor(
-                    iterator=vfi,
-                    record=record,  # <-- enable persistence
-                    session_factory=lambda: Session(database.engine),
-                    data_source_uuid=data_source_uuid,
-                    account_uuid=account_uuid,
-                    samples_per_file=samples_per_file,  # batch size
-                    writer_opts=writer_opts,  # forwarded to DataFile.save_local() writer
-                    content_type=recording_content_type,
+                    account_uuid=parameters.account_uuid,
+                    content_type=parameters.recording_content_type,
                     data_file_id=data_file_id,
+                    data_source_uuid=parameters.data_source_uuid,
+                    iterator=vfi,
+                    output_filename_template=parameters.output_filename_template,
+                    output_folder=parameters.output_folder,
+                    record=parameters.record,  # <-- enable persistence
+                    samples_per_file=parameters.samples_per_file,  # batch size
+                    session_factory=lambda: parameters.database.get_session(),
+                    writer_opts=writer_opts,  # forwarded to DataFile.save_local() writer
                 )
                 for data_sample in processor:
                     yield data_sample
@@ -876,22 +899,27 @@ class VideoDataSource(DataSourceCapability):
                 vfi = VideoReader(
                     source_buffer=source_buffer,
                     source_url=None,
-                    sample_fps=sample_fps,
+                    sample_fps=parameters.sample_fps,
+                    crop_zone=parameters.crop_zone,
                     resize_to=resize_to,
-                    output_type=self.output_type,
+                    original_fps=parameters.original_fps,
+                    open_opts=parameters.open_opts,
+                    output_type=parameters.output_type,
                     logger=self.logger,
                 )
-                writer_opts = {"frame_rate": vfi._fps, **writer_opts_overrides}
+                writer_opts = {"frame_rate": vfi._fps, **parameters.writer_opts}
                 processor = DataSampleProcessor(
-                    iterator=vfi,
-                    record=record,  # <-- enable persistence
-                    session_factory=lambda: Session(database.engine),
-                    data_source_uuid=data_source_uuid,
-                    account_uuid=account_uuid,
-                    samples_per_file=samples_per_file,  # batch size
-                    writer_opts=writer_opts,  # forwarded to DataFile.save_local()  writer
-                    content_type=recording_content_type,
+                    account_uuid=parameters.account_uuid,
+                    content_type=parameters.recording_content_type,
                     data_file_id=data_file_id,
+                    data_source_uuid=parameters.data_source_uuid,
+                    iterator=vfi,
+                    output_filename_template=parameters.output_filename_template,
+                    output_folder=parameters.output_folder,
+                    record=parameters.record,  # <-- enable persistence
+                    samples_per_file=parameters.samples_per_file,  # batch size
+                    session_factory=lambda: parameters.database.get_session(),
+                    writer_opts=parameters.writer_opts,  # forwarded to DataFile.save_local()  writer
                 )
                 for data_sample in processor:
                     yield data_sample
@@ -902,7 +930,6 @@ class VideoDataSource(DataSourceCapability):
             raise NotImplementedError("#### ToDo ####")
 
     def get_next_data_sample(self, stream):
-
         video_frame_generator = stream.variables.get("video_frame_generator", None)
         if video_frame_generator is None:
             video_frame_generator = self.get_video_frame_generator(stream)
