@@ -1,8 +1,6 @@
 from typing import Optional
 
-from ...core import GQLBaseModel
-
-__all__ = ["DatumSource"]
+from highlighter.core import GQLBaseModel
 
 
 class DatumSource(GQLBaseModel):
