@@ -30,6 +30,7 @@ from .gql_base import (
     to_gql_type,
     to_python_type,
 )
+from .json_tools import HLJSONEncoder
 
 LOGGER = logging.getLogger(".".join([__name__, "HLClient"]))
 
@@ -193,6 +194,8 @@ class HLClient(object):
                 "Content-Type": "application/json",
                 "Authorization": f"Token {api_token}",
             },
+            json_serialize=HLJSONEncoder().encode,
+            ssl=True,
         )
         client = GQLClient(
             transport=transport,
