@@ -1,7 +1,7 @@
 from .const import *
 from .decorators import *
 from .geometry import *
-from .hl_base_model import *
+from .gql_base_model import *
 from .itertools import *
 from .labeled_uuid import *
 from .pagination import *
