@@ -9,19 +9,20 @@ __all__ = [
 
 
 class Database:
-    """The Highlighter agent database"""
-
     def __init__(self):
-        # FIXME: How do we best pull HighlighterRuntimeConfig out of here?
         hl_cfg = HighlighterRuntimeConfig.load()
         self.highlighter_path_to_database_file = str(hl_cfg.agent.db_file())
-        SQLModel.metadata.create_all(self.engine)
+        self._engine = create_engine(
+            f"sqlite:///{self.highlighter_path_to_database_file}", connect_args={"check_same_thread": False}
+        )
+        SQLModel.metadata.create_all(self._engine)
 
     @property
     def engine(self) -> Engine:
-        return create_engine(
-            f"sqlite:///{self.highlighter_path_to_database_file}", connect_args={"check_same_thread": False}
-        )
+        return self._engine
 
     def get_session(self):
         return Session(self.engine)
+
+    def close(self):
+        self._engine.dispose()  # Call this when shutting down your app
