@@ -8,7 +8,7 @@ class ContentTypeEnum(str, Enum):  # inherit from str to support pydantic serial
     IMAGE = "image"
     JSON = "json"
     KML = "kml"
-    OBSERVATION = "observation"
+    ENTITIES = "entities"
     TEXT = "text"
     VIDEO = "video"
     WEB_PAGE = "web_page"
