@@ -10,3 +10,7 @@ def sha512_of_content(content):
 
 def get_slug(name) -> str:
     return re.sub(r"(?<!^)(?=[A-Z])", "_", name).lower()
+
+
+def stringify_if_not_null(attribute):
+    return str(attribute) if attribute is not None else None
