@@ -756,7 +756,7 @@ class Dataset:
         client: HLClient,
         dataset_id: int,
         datasets_cache_dir: Optional[Path] = None,
-        page_size: int = 200,
+        page_size=100,
         **kwargs,
     ):
         """Check for cached dataset"""
@@ -1177,6 +1177,7 @@ class Dataset:
         hl_client: HLClient,
         training_config: TrainingConfigType,
         dataset_dir: Path,
+        page_size=100,
     ) -> Dict[str, "Dataset"]:
 
         dataset_split_dict = {}
@@ -1210,6 +1211,7 @@ class Dataset:
                         dataset_format,
                         dataset_id=dataset_id,
                         client=hl_client,
+                        page_size=page_size,
                     )
                     ds.write_json(cache_path)
 
