@@ -15,7 +15,7 @@ from highlighter.core.exceptions import OptionalPackageMissingError
 try:
     import cv2
 except ModuleNotFoundError as _:
-    raise OptionalPackageMissingError("cv2", "opencv")
+    raise OptionalPackageMissingError("cv2", "cv2")
 
 try:
     import torch
@@ -42,7 +42,7 @@ class OnnxYoloV8:
         self,
         onnx_predictor: Union[OnnxPredictor, str, Path],
         num_classes: int,
-        class_lookup: Optional[Dict[int | str, Tuple[UUID, str]]] = None,
+        class_lookup: Optional[Dict[int | str, Tuple[UUID, str] | Tuple[UUID, str, float]]] = None,
         conf_thresh: float = 0.1,
         nms_iou_thresh: float = 0.5,
         is_absolute: bool = True,
@@ -52,7 +52,8 @@ class OnnxYoloV8:
         Args:
             num_classes: The number of output classes of the predictor
             class_lookup: Optional. If set then only classes in the dict will be returned. If not set the
-            the Annotation's ObjectClass Observation value with be UUID(int=class_id) with no 'human readable' label
+            the Annotation's ObjectClass Observation value with be UUID(int=class_id) with no 'human readable' label. An optional float
+            can be used to specify a per-class confidence thresh, if not it will fall back to the 'conf_thresh'
             conf_thresh: Only return detections with confidence >= conf_thresh
             nms_iou_thresh: Detections with an IOU >= nms_iou_thresh and the same class_id will be considered the same entity and the
             one with the max confidence will be used
@@ -69,17 +70,24 @@ class OnnxYoloV8:
         self.ndtype = (
             np.half if self._onnx_predictor.sess.get_inputs()[0].type == "tensor(float16)" else np.single
         )
-        self.nms_box_conf = conf_thresh  # Boxes with conf < this will be filterd out during NMS,
-        # keep this low because we filter out low conf preds in the Element
-
-        self.nms_iou_thr = nms_iou_thresh  # Boxes with iou < this will be filterd out during NMS
-
         self.num_classes = num_classes
-        self.class_lookup = None
+
         if class_lookup is not None:
             # Ensure keys are int so that class lookup works.
             # JSON doesn't allow numbers as keys so we may receive strings.
-            self.class_lookup = {int(k): v for k, v in class_lookup.items()}
+            # And, while we're at it we also any per-class confidence thresholds
+            # or default to conf_thresh
+            self.class_lookup = {}
+            for k, v in class_lookup.items():
+                if len(v) == 2:
+                    v = list(v) + [conf_thresh]
+                self.class_lookup[int(k)] = tuple(v)
+        else:
+            self.class_lookup = {i: (UUID(int=i), "", conf_thresh) for i in range(self.num_classes)}
+
+        self.nms_box_conf = min([v[2] for v in self.class_lookup.values()])
+        # Boxes with iou < this will be filterd out during NMS
+        self.nms_iou_thr = nms_iou_thresh
         self.is_absolute = is_absolute
 
         # Get model width and height(YOLOv8-seg only has one input)
@@ -127,7 +135,11 @@ class OnnxYoloV8:
             training_config = get_training_config(HLClient.get_client(), training_run_id)
 
         class_lookup = {
-            p.position: [p.taxon[0].entity_attribute_enum_id, p.taxon[0].entity_attribute_enum_value]
+            p.position: [
+                p.taxon[0].entity_attribute_enum_id,
+                p.taxon[0].entity_attribute_enum_value,
+                p.conf_thresh,
+            ]
             for p in training_config.input_output_schema.get_head_model_outputs(0)
         }
 
@@ -146,54 +158,78 @@ class OnnxYoloV8:
             img_pre = preprocess(img, self.model_height, self.model_width, self.ndtype)
             raw_ort_output = self._onnx_predictor.predict_batch(img_pre)
             if len(raw_ort_output[0].shape) == 2:
-                # classification  #TODO, make this work with any attribute
-                # not just object class
                 class_ids = np.argmax(raw_ort_output[0], axis=1)
-                if self.class_lookup is not None:
-                    preds = [self.class_lookup[class_id] for class_id in class_ids]
-                else:
-                    preds = [(UUID(int=class_id), "") for class_id in class_ids]
                 confs = np.max(raw_ort_output[0], axis=1)
-                image_annotations = [
-                    Annotation.from_left_top_right_bottom_box(
-                        (0, 0, img.shape[1], img.shape[0]),
-                        conf,
-                        observations=[
-                            Observation.make_object_class_observation(
-                                *pred,
-                                conf,
-                                datetime.now(),
-                            )
-                        ],
-                        data_file_id=data_samples[0].data_file_id,
-                        frame_id=data_sample.media_frame_index,
-                    )
-                    for conf, pred in zip(confs, preds)
-                ]
+                locations = [(0, 0) + data_sample.wh for _ in confs]
+                image_annotations = self._make_annotations(
+                    Annotation.from_left_top_right_bottom_box, class_ids, confs, locations, data_sample
+                )
+
             else:
                 # Need to fork here and run _postprocess depending if we're
                 # dealing with a classification responce or det/seg
                 image_annotations = self._postprocess(
                     raw_ort_output,
                     img_pre.shape[-2:],
-                    img.shape[:2],
-                    data_samples,
+                    data_sample,
                     self.is_absolute,
-                    frame_id=data_sample.media_frame_index,
                 )
 
             batch_output.append(image_annotations)
         return batch_output
 
+    # TODO: make this work with any attribute not just object class
+    def _make_annotations(self, make_anno_fn, class_ids, confs, locations, data_sample) -> List[Annotation]:
+        LOG.debug(f"{self.__class__.__name__} confs: {confs}")
+        LOG.debug(f"{self.__class__.__name__} class_ids: {class_ids}")
+
+        all_class_infos = [(i, self.class_lookup.get(class_id, None)) for i, class_id in enumerate(class_ids)]
+        invalid_idxs = np.array([i for i, object_class in all_class_infos if object_class is None])
+        class_infos = [id_name_thresh for i, id_name_thresh in all_class_infos if i not in invalid_idxs]
+        locations = [l for i, l in enumerate(locations) if i not in invalid_idxs]
+        confs = [c for i, c in enumerate(confs) if i not in invalid_idxs]
+
+        LOG.debug(
+            f"Dropping {len([i for i, object_class in all_class_infos if object_class is None])} detections with object class not in class_lookup"
+        )
+        LOG.debug(f"{self.__class__.__name__} locations: {locations}")
+        LOG.debug(f"{self.__class__.__name__} confs: {confs}")
+        LOG.debug(f"{self.__class__.__name__} class_infos: {class_infos}")
+
+        observations = [
+            Observation.make_object_class_observation(
+                cls_uuid,
+                cls_label,
+                conf,
+                frame_id=data_sample.media_frame_index,
+                occurred_at=data_sample.recorded_at,
+            )
+            for (cls_uuid, cls_label, cls_thr), conf in zip(class_infos, confs)
+            if conf >= cls_thr
+        ]
+
+        annotations = [
+            make_anno_fn(
+                loc,
+                obs.datum_source.confidence,
+                observations=[obs],
+                data_file_id=data_sample.data_file_id,
+                frame_id=data_sample.media_frame_index,
+                occurred_at=data_sample.recorded_at,
+            )
+            for loc, obs in zip(locations, observations)
+        ]
+        LOG.debug(f"{self.__class__.__name__} annotations: {annotations}")
+        return annotations
+
     def _postprocess(
         self,
         batch_ort_output,
         preproc_img_hw: Tuple[int, int],
-        orig_img_hw: Tuple[int, int],
-        data_samples,
+        data_sample,
         is_absolute: bool,
-        frame_id: int,
     ) -> List[Annotation]:
+        orig_img_hw = data_sample.wh[-1::-1]
 
         annotations: List[Annotation] = []
         if len(batch_ort_output) == 1:  # detection output
@@ -223,47 +259,12 @@ class OnnxYoloV8:
             trlb_boxes = preds[:, :4]
             confs = preds[:, 4]
             class_ids = preds[:, 5].astype(int)
-            LOG.debug(f"{self.__class__.__name__} confs: {confs}")
-            LOG.debug(f"{self.__class__.__name__} class_ids: {class_ids}")
 
-            if self.class_lookup is not None:
-                all_object_classes = [
-                    (i, self.class_lookup.get(class_id, None)) for i, class_id in enumerate(class_ids)
-                ]
-                valid_idxs = [i for i, object_class in all_object_classes if object_class is not None]
-                object_classes = [all_object_classes[i][1] for i in valid_idxs]
-                invalid_idxs = [i for i, object_class in all_object_classes if object_class is None]
-                LOG.debug(f"Dropping {len(invalid_idxs)} detections with object class not in class_lookup")
-                trlb_boxes = trlb_boxes[valid_idxs]
-                confs = confs[valid_idxs]
-            else:
-                object_classes = [(UUID(int=class_id), "") for class_id in class_ids]
-
-            LOG.debug(f"{self.__class__.__name__} preds (post scale_boxes): {preds[:, :4]}")
-            LOG.debug(f"{self.__class__.__name__} trlb_boxes: {trlb_boxes}")
-            LOG.debug(f"{self.__class__.__name__} confs: {confs}")
-            LOG.debug(f"{self.__class__.__name__} object_classes: {object_classes}")
+            annotations = self._make_annotations(
+                Annotation.from_left_top_right_bottom_box, class_ids, confs, trlb_boxes, data_sample
+            )
 
             # FIXME: DataSample, how do we associate Annotations to a data files
-            # when they may not exist yet.
-            annotations = [
-                Annotation.from_left_top_right_bottom_box(
-                    box,
-                    conf,
-                    observations=[
-                        Observation.make_object_class_observation(
-                            *object_class,
-                            conf,
-                            datetime.now(),
-                        )
-                    ],
-                    data_file_id=data_samples[0].data_file_id,
-                    frame_id=frame_id,
-                )
-                for box, object_class, conf in zip(trlb_boxes, object_classes, confs)
-            ]
-
-            LOG.debug(f"{self.__class__.__name__} annotations: {annotations}")
 
         elif len(batch_ort_output) == 2:  # segmentation output
             # Rename to make it easier to follow the reference implementation
@@ -298,22 +299,10 @@ class OnnxYoloV8:
                 confs = pred[:, 4].numpy()
                 class_ids = pred[:, 5].numpy()
 
-                annotations = [
-                    Annotation.from_mask(
-                        mask,
-                        conf,
-                        observations=[
-                            Observation.make_object_class_observation(
-                                UUID(int=class_id),
-                                "",
-                                conf,
-                                datetime.now(),
-                            )
-                        ],
-                        data_file_id=data_samples[0].data_file_id,
-                    )
-                    for mask, class_id, conf in zip(masks, class_ids.astype(int), confs)
-                ]
+                annotations = self._make_annotations(
+                    Annotation.from_mask, class_ids, confs, masks, data_sample
+                )
+
         else:
             raise ValueError("Invalid batch_ort_output")
 
