@@ -1,234 +1,29 @@
+import json
 import logging
-from abc import abstractmethod
-from copy import deepcopy
 from pathlib import Path
 from typing import Any, Dict, List, Optional
-from uuid import UUID
 
 import numpy as np
 import ultralytics
 import yaml
 
+from highlighter.client import TrainingConfigType, json_tools
 from highlighter.client.evaluation import (
     EvaluationMetric,
     EvaluationMetricCodeEnum,
+    EvaluationMetricResult,
     find_or_create_evaluation_metric,
 )
 from highlighter.client.gql_client import HLClient
 from highlighter.client.io import multithread_graphql_file_download
-from highlighter.client.training_config import TrainingConfigType
-from highlighter.core.const import OBJECT_CLASS_ATTRIBUTE_UUID
-from highlighter.core.labeled_uuid import LabeledUUID
 from highlighter.datasets.cropping import CropArgs
-from highlighter.datasets.dataset import Dataset
 from highlighter.datasets.formats.yolo.writer import YoloWriter
 from highlighter.trainers._scaffold import TrainerType
+from highlighter.trainers.base_trainer import BaseTrainer
 
 __all__ = ["YoloV11Trainer"]
 
 
-class BaseTrainer:
-
-    head_idx: int = 0
-
-    crop_args: CropArgs = CropArgs()
-
-    # Only use attributes with this specific attribute_id as the
-    # categories for your dataset
-    category_attribute_id: UUID = OBJECT_CLASS_ATTRIBUTE_UUID
-
-    # Optionally define a list of attribute values to use for your dataset.
-    # If None, then use the output attributes defined in the TrainingConfigType.input_output_schema.
-    # If categories is set then the YoloWrite will respect the order they are
-    # listed.
-    categories: Optional[List[str]] = None
-
-    def __init__(
-        self,
-        training_run_dir: Path,
-        highlighter_training_config: TrainingConfigType,
-        trainer_type: TrainerType,
-    ):
-        self._trainer_type = trainer_type
-
-        self._logger = logging.getLogger(__name__)
-        self._training_run_dir = training_run_dir
-        self._hl_training_config = highlighter_training_config
-        self._hl_training_config.dump("json", self._hl_training_config_path)
-        with (self._hl_cache_dir / "trainer-type").open("w") as f:
-            f.write(trainer_type.value)
-
-    @property
-    def _hl_cache_dir(self):
-        d = self._training_run_dir / ".hl"
-        d.mkdir(exist_ok=True, parents=True)
-        return d
-
-    @property
-    def _hl_cache_dataset_dir(self):
-        d = self._hl_cache_dir / "datasets"
-        d.mkdir(exist_ok=True, parents=True)
-        return d
-
-    @property
-    def _hl_training_config_path(self):
-        d = self._hl_cache_dir / "training_config.json"
-        return d
-
-    @abstractmethod
-    def generate_boilerplate(self):
-        """Puts the standard boilerplate files and makes directories required
-        by in the Trainer in the training_run_dir. Downloads as caches
-        the dataset annotations
-        """
-        pass
-
-    @abstractmethod
-    def training_data_dir(self) -> Path:
-        """Path to data used to store training data"""
-
-    @abstractmethod
-    def generate_trainer_specific_dataset(self, hl_dataset: Dataset):
-        pass
-
-    @abstractmethod
-    def train(self) -> Any:
-        """Trains the Model, return the trained model instance"""
-        pass
-
-    @abstractmethod
-    def evaluate(self, checkpoint: Path, object_classes: List[LabeledUUID], cfg_path: Optional[Path] = None):
-        """Evaluate a model given a Path to a checkpoint, onnx_file or model instance"""
-        pass
-
-    @abstractmethod
-    def export_to_onnx(self, trained_model) -> Path:
-        """Export the trained model to onnx
-
-        Return:
-            Path to the onnx model
-        """
-        pass
-
-    @abstractmethod
-    def make_artefact(self, onnx_file_path: Path) -> Path:
-        """Create a Highlighter training run artefact give the input checkpoint
-
-        Return:
-            Path to the artefact.yaml
-        """
-        pass
-
-    @property
-    def training_run_id(self) -> int:
-        return self._hl_training_config.training_run_id
-
-    @property
-    def research_plan_id(self) -> int:
-        return self._hl_training_config.evaluation_id
-
-    def get_datasets(self, client: HLClient) -> Dataset:
-        """Returns a Highlighter SDK Dataset object containing data from each
-        split in the one object. `dataset.data_files_id.split` identifies the
-        which split each data_file belongs to.
-        """
-        datasets = Dataset.read_training_config(client, self._hl_training_config, self._hl_cache_dataset_dir)
-        return self._combine_hl_datasets(datasets)
-
-    def _combine_hl_datasets(self, datasets):
-        # When creating a training run in Highlighter the train split is required
-        # but a user can supply either a test or dev set, or both. If not both we
-        # duplicate the one that exists here
-        if "test" not in datasets:
-            datasets["test"] = deepcopy(datasets["dev"])
-            datasets["test"].data_files_df.split = "test"
-        if "dev" not in datasets:
-            datasets["dev"] = deepcopy(datasets["test"])
-            datasets["dev"].data_files_df.split = "dev"
-
-        # Combine the Highlighter Datasets together because this is what the YoloWriter
-        # expects
-        combined_ds = datasets["train"]
-        combined_ds.append([datasets["dev"], datasets["test"]])
-
-        # Ultralytics name their dataset splits differently so
-        # we need to map them
-        #   their "val" is our "test"
-        #   their "test" is our "dev"
-        combined_ds.data_files_df.loc[combined_ds.data_files_df.split == "test", "split"] = "val"
-        combined_ds.data_files_df.loc[combined_ds.data_files_df.split == "dev", "split"] = "test"
-        return combined_ds
-
-    def filter_dataset(self, dataset: Dataset) -> Dataset:
-        """Optionally add some code to filter the Highlighter Datasets as required.
-        The YoloWriter will only use entities with both a pixel_location attribute
-        and a 'category_attribute_id' attribute when converting to the Yolo dataset format.
-        It will the unique values for the object_class attribute as the detection
-        categories.
-
-        For example, if you want to train a detector that finds Apples and Bananas,
-        and your taxonomy looks like this:
-
-            - object_class: Apple
-            - object_class: Orange
-            - object_class: Banana
-
-        Then you may do something like this:
-
-            adf = combined_ds.annotations_df
-            ddf = combined_ds.data_files_df
-
-            orange_entity_ids = adf[(adf.attribute_id == OBJECT_CLASS_ATTRIBUTE_UUID) &
-                                   (adf.value == "Orange")].entity_id.unique()
-
-            # Filter out offending entities
-            adf = adf[adf.entity_id.isin(orange_entity_ids)]
-
-            # clean up images that are no longer needed
-            ddf = ddf[ddf.data_file_id.isin(adf.data_file_id)]
-
-            combined_ds.annotations_df = adf
-        """
-        return dataset
-
-    def _train(self, hl_dataset: Dataset):
-
-        self.generate_trainer_specific_dataset(hl_dataset)
-
-        trained_model = self.train()
-        onnx_model_path = self.export_to_onnx(trained_model)
-        artefact_path = self.make_artefact(onnx_model_path)
-        self.evaluate(onnx_model_path, self.get_categories())
-        return trained_model, artefact_path.absolute(), onnx_model_path
-
-    def get_categories(self) -> List[LabeledUUID]:
-        all_cat_ids = self._hl_training_config.input_output_schema.get_head_output_attribute_enum_ids(
-            self.head_idx
-        )
-        all_cat_values = self._hl_training_config.input_output_schema.get_head_output_attribute_enum_values(
-            self.head_idx
-        )
-
-        if self.categories is None:
-            categories = self._hl_training_config.input_output_schema.get_head_output_attribute_enum_ids(
-                self.head_idx
-            )
-        else:
-            categories = self.categories
-
-        idxs = [all_cat_ids.index(c) for c in categories]
-        return [LabeledUUID.from_str(f"{all_cat_ids[i]}|{all_cat_values[i]}") for i in idxs]
-
-    def get_category_attribute_id(self):
-        if self.category_attribute_id is None:
-            category_attribute_id = (
-                self._hl_training_config.input_output_schema.get_head_output_attribute_ids(self.head_idx)[0]
-            )
-        else:
-            category_attribute_id = self.category_attribute_id
-        return category_attribute_id
-
-
 class YoloV11Trainer(BaseTrainer):
 
     def __init__(
@@ -240,6 +35,17 @@ class YoloV11Trainer(BaseTrainer):
         super().__init__(training_run_dir, highlighter_training_config, trainer_type)
         self._cfg = self._get_config()
 
+    def _combine_hl_datasets(self, datasets):
+        combined_ds = super()._combine_hl_datasets(datasets)
+
+        # Ultralytics name their dataset splits differently so
+        # we need to map them
+        #   their "val" is our "test"
+        #   their "test" is our "dev"
+        combined_ds.data_files_df.loc[combined_ds.data_files_df.split == "test", "split"] = "val"
+        combined_ds.data_files_df.loc[combined_ds.data_files_df.split == "dev", "split"] = "test"
+        return combined_ds
+
     def _get_hl_metrics(self, client: HLClient):
 
         task = self._cfg["task"]
@@ -525,8 +331,8 @@ class YoloV11Trainer(BaseTrainer):
 
         self._cfg["model"] = checkpoint
         self._cfg.update(cfg_overrides)
-        model = ultralytics.YOLO(self._cfg["model"])
-        artefact_path, _ = self.make_artefact(model)
+        model = Path(self._cfg["model"])
+        artefact_path = self.make_artefact(model)
 
         return artefact_path.absolute()
 
@@ -557,6 +363,7 @@ class YoloV11Trainer(BaseTrainer):
 
             ddf = filtered_hl_ds.data_files_df
             if any([Path(f).suffix.lower() == ".mp4" for f in ddf.filename.unique()]):
+
                 print("Detected video dataset, interpolating data from keyframes")
                 filtered_hl_ds = filtered_hl_ds.interpolate_from_key_frames(
                     frame_save_dir=image_cache_dir,
@@ -574,11 +381,31 @@ class YoloV11Trainer(BaseTrainer):
             )
             writer.write(filtered_hl_ds)
 
-    def evaluate(self, checkpoint: Path, object_classes: List[str], cfg_path: Optional[Path] = None):
+    def evaluate(
+        self, checkpoint: Path | str, object_classes: List[str], cfg_path: Optional[Path | str] = None
+    ) -> Dict[str, EvaluationMetricResult]:
+        """Evaluate a YOLOv11 model and generate structured evaluation metrics.
+
+        Evaluates the model using ultralytics validation and creates evaluation metric
+        result objects for upload to Highlighter. Handles both classification and
+        detection/segmentation tasks with appropriate metrics for each.
+
+        Args:
+            checkpoint: Path to model checkpoint (can be .pt, .onnx, or other supported formats)
+            object_classes: List of object class names for the model
+            cfg_path: Optional path to configuration file (uses self._cfg if None)
+
+        Returns:
+            List[EvaluationMetricResult]: Evaluation metric results ready for upload
+
+        Side Effects:
+            Saves evaluation results to 'eval_metric_results.json' in checkpoint directory
+        """
+        eval_metric_results: Dict[str, EvaluationMetricResult] = {}
         if cfg_path is None:
             cfg: Dict = self._cfg
         else:
-            cfg: Dict = ultralytics.utils.YAML.load(cfg_path)
+            cfg: Dict = ultralytics.utils.YAML.load(str(cfg_path))
 
         assert cfg["data"] is not None
         client = HLClient.get_client()
@@ -587,40 +414,98 @@ class YoloV11Trainer(BaseTrainer):
         results = model.val(**cfg)
 
         if cfg["task"] in ("segment", "detect"):
-            self._create_eval_results_det_or_seg(results, cfg, client)
+            eval_metric_results = self._create_eval_results_det_or_seg(results, cfg, client)
         elif cfg["task"] == "classify":
-            self._create_eval_results_cls(results, cfg, client)
+            eval_metric_results = self._create_eval_results_cls(results, cfg, client)
 
-        return results
+        with (Path(checkpoint).parent / "eval_metric_results.json").open("w") as f:
+            json.dump(
+                {k: e.model_dump() for k, e in eval_metric_results.items()},
+                f,
+                indent=2,
+                cls=json_tools.HLJSONEncoder,
+            )
 
-    def _create_eval_results_cls(self, results, cfg: Dict, client: HLClient):
+        return eval_metric_results
+
+    def _create_eval_results_cls(
+        self, results, cfg: Dict, client: HLClient
+    ) -> Dict[str, EvaluationMetricResult]:
+        """Create evaluation metric results for classification tasks.
+
+        Extracts classification-specific metrics including model size, top-1 accuracy,
+        and top-5 accuracy from ultralytics validation results.
+
+        Args:
+            results: Ultralytics validation results object
+            cfg: Configuration dictionary containing model information
+            client: HLClient for metric lookup
+
+        Returns:
+            List[EvaluationMetricResult]: Classification evaluation metrics
+        """
+        eval_metric_results: Dict[str, EvaluationMetricResult] = {}
         _hl_metrics = self._get_hl_metrics(client)
 
         model_size_char = Path(cfg["model"]).stem.replace("-cls", "")[-1]
         if model_size_char in "nsmlx":
             model_size_int = "nsmlx".index(model_size_char)
-            _hl_metrics["Model Size"].create_result(client, model_size_int, self.training_run_id)
+            eval_metric_results["Model Size"] = _hl_metrics["Model Size"].result(
+                model_size_int, self.training_run_id
+            )
         else:
             model_str = cfg["model"]
             raise SystemExit(f"Unvalid model_size_char '{model_size_char}' from '{model_str}'")
         acc_top1 = results.results_dict["metrics/accuracy_top1"]
         acc_top5 = results.results_dict["metrics/accuracy_top5"]
-        _hl_metrics["Accuracy Top1"].create_result(client, acc_top1, self.training_run_id)
-        _hl_metrics["Accuracy Top5"].create_result(client, acc_top5, self.training_run_id)
+        eval_metric_results["Accuracy Top1"] = _hl_metrics["Accuracy Top1"].result(
+            acc_top1, self.training_run_id
+        )
+        eval_metric_results["Accuracy Top5"] = _hl_metrics["Accuracy Top5"].result(
+            acc_top5, self.training_run_id
+        )
+        return eval_metric_results
 
-    def _create_eval_results_det_or_seg(self, results, cfg: Dict, client: HLClient):
-        """Evaluate the Yolo model, will work with Torch and Onnx weights"""
+    def _create_eval_results_det_or_seg(
+        self, results, cfg: Dict, client: HLClient
+    ) -> Dict[str, EvaluationMetricResult]:
+        """Create evaluation metric results for detection and segmentation tasks.
+
+        Extracts detection/segmentation-specific metrics including precision, recall,
+        mAP@IoU50, model size, ideal confidence threshold, and per-class mAP values
+        from ultralytics validation results.
+
+        Args:
+            results: Ultralytics validation results object
+            cfg: Configuration dictionary containing model information
+            client: HLClient for metric lookup
+
+        Returns:
+            List[EvaluationMetricResult]: Detection/segmentation evaluation metrics
+
+        Note:
+            Uses corrected ultralytics result attributes (e.g., 'metrics/precision(B)')
+            rather than deprecated mean_results() method for better accuracy.
+        """
+        eval_metric_results: Dict[str, EvaluationMetricResult] = {}
         _hl_metrics = self._get_hl_metrics(client)
 
-        agg_precision, agg_recall, agg_map50 = results.mean_results()[:3]
-        _hl_metrics["Precision"].create_result(client, agg_precision, self.training_run_id)
-        _hl_metrics["Recall"].create_result(client, agg_recall, self.training_run_id)
-        _hl_metrics["mAP@IOU50"].create_result(client, agg_map50, self.training_run_id)
+        eval_metric_results["Precision"] = _hl_metrics["Precision"].result(
+            results.results_dict["metrics/precision(B)"], self.training_run_id
+        )
+        eval_metric_results["Recall"] = _hl_metrics["Recall"].result(
+            results.results_dict["metrics/recall(B)"], self.training_run_id
+        )
+        eval_metric_results["mAP@IOU50"] = _hl_metrics["mAP@IOU50"].result(
+            results.results_dict["metrics/mAP50(B)"], self.training_run_id
+        )
 
         model_size_char = Path(cfg["model"]).stem.replace("-det", "").replace("-seg", "")[-1]
         if model_size_char in "nsmlx":
             model_size_int = "nsmlx".index(model_size_char)
-            _hl_metrics["Model Size"].create_result(client, model_size_int, self.training_run_id)
+            eval_metric_results["Model Size"] = _hl_metrics["Model Size"].result(
+                model_size_int, self.training_run_id
+            )
         else:
             model_str = cfg["model"]
             raise SystemExit(f"Unvalid model_size_char '{model_size_char}' from '{model_str}'")
@@ -632,28 +517,17 @@ class YoloV11Trainer(BaseTrainer):
         else:
             best_f1_idx = np.argmax(results.curves_results[f1_conf_curve_idx][1])
         best_f1_thr = results.curves_results[f1_conf_curve_idx][0][best_f1_idx]
-        _hl_metrics["Ideal Confidence Threshold"].create_result(client, best_f1_thr, self.training_run_id)
+        eval_metric_results["Ideal Confidence Threshold"] = _hl_metrics["Ideal Confidence Threshold"].result(
+            best_f1_thr, self.training_run_id
+        )
 
         cats = self.get_categories()
-        for cls_idx, _ in results.names.items():
-            class_name = cats[cls_idx].short_str()
-            _hl_metrics[f"mAP@IOU50({class_name})"].create_result(
-                client, results.maps[cls_idx], self.training_run_id
+        CLS_PRECISION_IOU50 = 2
+        for cls_idx, cat in enumerate(cats):
+            class_name = cat.short_str()
+            metric_name = f"mAP@IOU50({class_name})"
+            eval_metric_results[metric_name] = _hl_metrics[metric_name].result(
+                results.class_result(cls_idx)[CLS_PRECISION_IOU50], self.training_run_id
             )
 
-        return results
-
-
-if __name__ == "__main__":
-    _ = HLClient.from_profile("ci")
-    obj = [LabeledUUID(int=1, label="cat"), LabeledUUID(int=2, label="dog")]
-    t = YoloV11Trainer(
-        Path("/home/josh/clients/energy-queensland/pole_top_assembly/ml_training/645/"), None, obj
-    )
-
-    t.evaluate(
-        Path(
-            "/home/josh/clients/energy-queensland/pole_top_assembly/ml_training/645/runs/train/weights/best.onnx"
-        ),
-        Path("/home/josh/clients/energy-queensland/pole_top_assembly/ml_training/645/runs/train/args.yaml"),
-    )
+        return eval_metric_results
